import { DialogService, LoadingService, Modal } from '@zeedhi/common';
import { Http } from '@zeedhi/core';
import { AlertServiceCustom } from './AlertServiceCustom';
import { DialogServiceCustom } from './DialogServiceCustom';
import { InstanceService } from './InstanceService';

export class Utilities {
    private static deleteRoute: string = '';

    private static deleteTekGridName: string = '';

    public static getNameDynamic(name: string) {
        const date = new Date();
        return name + date.getTime();
    }

    public static async genericAddModalTekGrid(formName: string, modal: Modal, route: string, tekGridName: string, clearDataSourceFilter: boolean = false) {
        const form = InstanceService.getInstanceZdForm(formName);
        const grid = InstanceService.getInstanceTekGrid(tekGridName);
		if (form.validate()) {
            try {
                LoadingService.show();
                await Http.post(route, form.value);
                LoadingService.hide();
                modal.hide();
                AlertServiceCustom.defaultSaveSuccessAlert();
                if (clearDataSourceFilter) {
                    grid.datasource.clearFilter();
                }
                grid.datasource.get();
            } catch (e) {
                LoadingService.hide();
                DialogServiceCustom.showErrorBackend(e);
            }
        }
    }

    public static async genericBeforeSaveTekGrid(params: any, route: string, tekGridName: string) {
        params.event.preventDefault();
        try {
            LoadingService.show();
            await Http.post(route, params.component.getEditedRows());
            LoadingService.hide();
            AlertServiceCustom.defaultSaveSuccessAlert();
            params.component.cancelEditedRows();
            InstanceService.getInstanceTekGrid(tekGridName).datasource.get();
        } catch (e) {
            LoadingService.hide();
            DialogServiceCustom.showErrorBackend(e);
        }
    }

    public static async genericBeforeDeleteTekGrid(params: any, route: string, tekGridName: string) {
        params.event.preventDefault();
        this.deleteRoute = route;
        this.deleteTekGridName = tekGridName;

        DialogService.show({
			name: `confirm_${tekGridName}`,
			type: 'warning',
			title: 'Confirmação de Exclusão',
			text: 'Deseja realmente excluir os registros selecionados?',
			buttons: [
				{
                    name: 'no',
                    component: 'ZdButton',
                    label: 'Não',
                    outline: true,
                    events: {
                        click: () => DialogService.hide(),
                    },
                },
				{
                    name: 'yes',
                    component: 'ZdButton',
                    label: 'Sim',
                    events: Utilities.deleteTekGrid,
                },
			],
		});
    }

    private static deleteTekGrid: any = {
        click: () => Utilities.doDeleteTekGrid(),
    };

    private static async doDeleteTekGrid() {
        DialogService.hide();
        try {
            const grid = InstanceService.getInstanceTekGrid(this.deleteTekGridName);
            LoadingService.show();
            await Http.post(this.deleteRoute, grid.selectedRows);
            LoadingService.hide();
            AlertServiceCustom.defaultDeleteSuccessAlert();
            grid.datasource.get();
            grid.selectedRows = [];
        } catch (e) {
            LoadingService.hide();
            DialogServiceCustom.showErrorBackend(e);
        }
        this.deleteRoute = '';
        this.deleteTekGridName = '';
    }
}
